# -*- coding: utf-8 -*-
from setuphelpers import *
import waptguihelper
import webbrowser
import os
import sys

if "__file__" in locals():
    sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)))
from setupdevhelpers import *


def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    extract_path = "extract"

    # if not the first run it will use the last setup downloaded if you want to get an updated one you can remove setup.exe and relaunch the update package
    if not isfile("setup.exe") and not params.get("running_as_luti"):
        waptguihelper.message_dialog(
            "You browser will open",
            "Please download Office Deployment Tool by clicking the download button",
        )
        webbrowser.open("https://www.microsoft.com/download/details.aspx?id=49117")
        waptguihelper.message_dialog("Waiting for download", "Click OK when the download is complete", waptguihelper.ID_OK)
        # Copy to pkg
        latest_bin = waptguihelper.filename_dialog(
            "Please provide downloaded Office Deployment Tool to copy in this package",
            makepath(user_home_directory, "Downloads"),
            "",
            "EXE Files|*.exe",
        )

        # Extract Office Deployment Tool
        print("Copying: " + latest_bin)
        run(rf'"{latest_bin}" /extract:"{extract_path}" /quiet"')

        print("Extract Office Deployment Tool")
        filecopyto(".\\extract\\setup.exe", basedir)
        remove_file(latest_bin)
        if isdir("extract"):
            remove_tree("extract")

    elif params.get("running_as_luti"):
        download_url = "https://download.microsoft.com/download/2/7/A/27AF1BE6-DD20-4CB4-B154-EBAB8A7D4A7E/officedeploymenttool_16731-20354.exe"
        latest_bin = download_url.split("/")[-1]
        wget(download_url, latest_bin, proxies=proxies)

        # Extract Office Deployment Tool
        print("Copying: " + latest_bin)
        run(rf'"{latest_bin}" /extract:"{extract_path}" /quiet"')

        print("Extract Office Deployment Tool")
        filecopyto(".\\extract\\setup.exe", basedir)
        remove_file(latest_bin)
        if isdir("extract"):
            remove_tree("extract")

    # if not the first run it will use the last XML used if you want to get a new one you can remove configuration.xml and relaunch the update package
    if not isfile("configuration.xml"):
        waptguihelper.message_dialog(
            "Your browser will open",
            "Please complete your deployment settings with the online Office Customization Tool and download it by clicking Export.\nNote that you can go back anytime on the online tool to edit your configuration (by clicking Import)",
        )
        webbrowser.open("https://config.office.com/deploymentsettings")
        waptguihelper.message_dialog("Waiting for download", "Click OK when the XML is completed and downloaded", waptguihelper.ID_OK)
        xml_configuration = waptguihelper.filename_dialog(
            "Please select your XML configuration file", basedir, "configuration.xml", "XML Files|*.xml"
        )  # makepath(user_home_directory, "Downloads")
        if not isfile("configuration.xml"):
            filecopyto(
                xml_configuration,
                basedir + "\\configuration.xml",
            )

    # Asking to remove the last downloaded sources
    if isdir("Office"):
        if not params.get("running_as_luti"):
            response = ask_message(control.package, "Do you want to remove the last downloaded sources?", 35)
            if response == 6:
                print("Removing last sources folder downloaded")
                remove_tree("Office")
        else:
            print("Removing last sources folder downloaded")
            remove_tree("Office")
    else:
        # Download Office with XML configuration
        print("Downloading Office with XML configuration")
        if windows_version() >= WindowsVersions.Windows10:
            run("setup.exe /download configuration.xml", timeout=2400)  # DisplayLevel=Full # not working
        else:
            error("MSO setup.exe can no longer be run on this OS")

    # Getting version from source dir
    version = glob.glob("Office/Data/**/")[0].split(os.sep)[1]

    # Changing version of the package
    if Version(version, 4) > Version(control.get_software_version(), 4):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Getting Product ID and Language ID from configuration.xml parsing file
    with open("configuration.xml", "r", encoding="utf8") as f:
        for line in f:
            if "OfficeClientEdition" in line:
                architecture = line.split('"')[1]
                print("OfficeClientEdition: %s" % architecture)
                if architecture == "64":
                    architecture = "x64"
                else:
                    architecture = "all"
            if "Product ID" in line:
                product_id = line.split('"')[1]
                print("Product ID: %s" % product_id)
            if "Language ID" in line:
                language_id = line.split('"')[1]
                print("Language ID: %s" % language_id)
                break

    if control.architecture != architecture:
        control.architecture = architecture
        control.save_control_to_wapt()

    # Asking pkg infos if needed
    if control.locale not in language_id and "-template" in control.package:
        control.locale = waptguihelper.input_dialog(control.package, "You may wanna change locale to: %s" % language_id.split("-")[0], "all")
        print("Changing locale to: %s in WAPT\\control" % control.locale)
        control.save_control_to_wapt()

    complete_control_categories(control)
    complete_control_name(control, control.name, params.get("running_as_luti"))
    complete_control_package(control, control.package, params.get("running_as_luti"), False)
    complete_control_description(control, control.description, params.get("running_as_luti"), "update_package")

    # changing uninstallkey:
    new_lines = []
    with open("setup.py", "r", encoding="utf8") as f:
        for line in f.readlines():
            if line.startswith("app_uninstallkey"):
                line = f'app_uninstallkey = "{product_id} - {language_id}"\n'
                print(f'uninstallkey in setup updated to app_uninstallkey = "{product_id} - {language_id}"\n')
            new_lines.append(line)
    with open("setup.py", "w", encoding="utf8", newline="\n") as f:
        f.writelines(new_lines)

    # Validating or not update-package-sources
    return package_updated
